@@ -44,7 +44,7 @@
 #include "quit.h"
 #include "trap.h"
 
-#if __CYGWIN__
+#ifdef __CYGWIN__
 int igncr;
 #endif
 
@@ -276,7 +276,7 @@ save_bash_input (fd, new_fd)
       return -1;
     }
 
-  if (buffers[nfd])
+  if (nfd < nbuffers && buffers[nfd])
     {
       /* What's this?  A stray buffer without an associated open file
 	 descriptor?  Free up the buffer and report the error. */
@@ -565,10 +565,10 @@ buffered_getchar ()
 {
   CHECK_TERMSIG;
 
-#if __CYGWIN__
-  /* shopt igncr means to discard carriage returns from input stream.
-     If cr is the only character in the buffer, then recurse to pick
-     up the next character; otherwise flatten the buffer.  */
+#ifdef __CYGWIN__
+  /* shopt igncr discards carriage returns from the input stream.
+     If cr is the only character left in the buffer, recurse to pick
+     up the next byte; otherwise flatten the buffer.  */
   if (igncr)
     {
       int ch;
@@ -577,7 +577,7 @@ buffered_getchar ()
 	;
       return ch;
     }
-#endif /* __CYGWIN__ */
+#endif
 #if !defined (DJGPP)
   return (bufstream_getc (buffers[bash_input.location.buffered_fd]));
 #else
